/*=====================================================================
 * Any authorized distribution of any copy of this code (including any
 * related documentation) must reproduce the following restrictions,
 * disclaimer and copyright notice:
 * 
 * The Genesys name, the trademarks and/or logo(s) of Genesys shall
 * not be used to name (even as a part of another name), endorse and/or
 * promote products derived from this code without prior written
 * permission from Genesys Telecommunications Laboratories, Inc.
 * 
 * The use, copy, and/or distribution of this code is subject to the
 * terms of the Genesys Developer License Agreement.  This code shall
 * not be used, copied, and/or distributed under any other license
 * agreement.
 * 
 * THIS CODE IS PROVIDED BY GENESYS TELECOMMUNICATIONS LABORATORIES,
 * INC. (GENESYS) AS IS WITHOUT ANY WARRANTY OF ANY KIND. GENESYS
 * HEREBY DISCLAIMS ALL EXPRESS, IMPLIED, OR STATUTORY CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES WITH RESPECT TO THIS CODE (OR ANY
 * PART THEREOF), INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT. GENESYS AND ITS SUPPLIERS SHALL NOT BE LIABLE FOR
 * ANY DAMAGE SUFFERED AS A RESULT OF USING THIS CODE. IN NO EVENT
 * SHALL GENESYS AND ITS SUPPLIERS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * CONSEQUENTIAL, ECONOMIC, INCIDENTAL, OR SPECIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, ANY LOST REVENUES OR PROFITS).
 * 
 * Copyright (C) 2006-2008 Genesys Telecommunications Laboratories,
 * Inc. All rights reserved.
 =====================================================================*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.IO;
using Genesyslab.Platform.ApplicationBlocks.Commons.Broker;
using Genesyslab.Platform.Commons.Collections;
using Genesyslab.Platform.Commons.Connection;
using Genesyslab.Platform.Commons.Protocols;
using Genesyslab.Platform.Configuration.Protocols;
using Genesyslab.Platform.Configuration.Protocols.ConfServer;
using Genesyslab.Platform.Configuration.Protocols.ConfServer.Events;
using Genesyslab.Platform.Configuration.Protocols.ConfServer.Requests.Objects;
using Genesyslab.Platform.ApplicationBlocks.Commons.Protocols;

namespace ConfigPlatformSDKExample
{
    public partial class Form1 : Form
    {
        // Enter configuration information here:
        private string confServerHost = "<host>";
        private int confServerPort = <port>;
        private string clientName = "<clientName>";
        private string userName = "<userName>";
        private string password = "<password>";
        // End of configuration information.

        private const string protocolName = "ConfServer";
        private const string sdkName = "Configuration";

        private Uri confServerUri;

        EventBrokerService eventBrokerService;
        ProtocolManagementService protocolManagementService;

        public Form1()
        {
            InitializeComponent();
            InitializeServices();
        }

        private void InitializeServices()
        {
            confServerUri = new Uri("tcp://" + confServerHost + ":" + confServerPort);

            protocolManagementService = new ProtocolManagementService();
            ConfServerConfiguration confServerConfiguration = new ConfServerConfiguration("Config_Server_App");
            confServerConfiguration.Uri = confServerUri;
            confServerConfiguration.ClientType = ConfServerClientType.SCE;
            confServerConfiguration.ClientName = clientName;
            confServerConfiguration.UserName = userName;
            confServerConfiguration.UserPassword = password;
            protocolManagementService.Register(confServerConfiguration);

            eventBrokerService =
                 BrokerServiceFactory.CreateEventBroker(protocolManagementService.Receiver);

            eventBrokerService.Register(this.OnEventObjectsRead, new MessageIdFilter(EventObjectsRead.MessageId));
            eventBrokerService.Register(this.OnEventObjectsSent, new MessageIdFilter(EventObjectsSent.MessageId));
            eventBrokerService.Register(this.OnEventError, new MessageIdFilter(EventError.MessageId));

            protocolManagementService["Config_Server_App"].Open();
        }

        private void OnEventError(IMessage theMessage)
        {
            EventError eventError = theMessage as EventError;
            if (eventError == null)
            {
                writeToLogArea("EventError!\n");
                return;
            }

            writeToLogArea("EventError:\n"
                    + eventError + "\n\n");
        }

        private void OnEventObjectsSent(IMessage theMessage)
        {
            EventObjectsSent objectsSent = theMessage as EventObjectsSent;
            if (objectsSent == null)
            {
                writeToLogArea("EventObjectsSent!\n");
                return;
            }

            writeToLogArea("EventObjectsSent:\n"
                    + objectsSent + "\n\n");
        }

        private void OnEventObjectsRead(IMessage theMessage)
        {
            EventObjectsRead objectsRead = theMessage as EventObjectsRead;
            if (objectsRead == null)
            {
                writeToLogArea
                    ("There are no objects of the requested type\nin the Genesys Configuration Layer.\n\n");
                return;
            }

            XmlDocument resultDocument = new XmlDocument();
            resultDocument =
                (XmlDocument)objectsRead.ConfObject;
            StringBuilder xmlAsText = new StringBuilder();
            StringWriter stringWriter = new StringWriter(xmlAsText);
            XmlTextWriter xmlTextWriter =
                new XmlTextWriter(stringWriter);
            xmlTextWriter.Formatting = Formatting.Indented;

            resultDocument.WriteContentTo(xmlTextWriter);

            writeToLogArea("This is the response:\n\n"
                    + xmlAsText.ToString() + "\n\n");
        }

        private void ReadObjectsButton_Click(object sender, EventArgs e)
        {
            ReadObjectsButton.Enabled = false;

            KeyValueCollection filterKey = new KeyValueCollection();
            filterKey.Add("user_name", userName);
            RequestReadObjects requestReadObjects =
                RequestReadObjects.Create(
                    (int)ConfServerObjectType.Filter,
                    filterKey);
            protocolManagementService["Config_Server_App"].Send(requestReadObjects);
        }

        /// <summary>
        /// Writes the specified text to the logging pane at the bottom of the user
        /// interface.
        /// </summary>
        /// <param name="messageText">The text to write to the logging pane</param>
        private void writeToLogArea(string messageText)
        {
            UIwriteToLogArea(messageText, EventArgs.Empty);

            //			System.Diagnostics.Debug.WriteLine(messageText);
        }

        private delegate void UIEventHandler(object sender, EventArgs eventArgs);

        private void UIwriteToLogArea(object sender, System.EventArgs eventArgs)
        {

            if (this.InvokeRequired)
            {

                this.BeginInvoke(new UIEventHandler(this.UIwriteToLogArea), new object[] { sender, eventArgs });

                return;

            }

            string text = (string)sender;

            LogAreaRichTextBox.Text = LogAreaRichTextBox.Text + text;
            LogAreaRichTextBox.Focus();
            LogAreaRichTextBox.SelectionStart = LogAreaRichTextBox.TextLength;
            LogAreaRichTextBox.SelectionLength = 0;
        }
    }
}